// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund.Components.BioDiversity;
// Last change: 17 April 2008


interface

uses Esmf;

type
  
  /// <summary>State for the biodiversity component</summary>
  IBioDiversityState = public interface
    /// <summary> Number of species </summary>
    property nospecies: IVariable1Dimensional<Timestep, double> read;
       
    /// <summary> additive parameter </summary>
    property bioloss: double read;
    
    /// <summary> multiplicative parameter </summary>
    property biosens: double read;
    
    /// <summary> Temperature </summary>
    property temp: IParameter1Dimensional<Timestep, double> read;
    
    /// <summary> benchmark temperature change </summary>
    property dbsta: double read;
    
    /// <summary> Number of species in the year 2000 </summary>
    property nospecbase: double read;    
  end;  
  
  /// <summary>Biodiversity component</summary>
  BioDiversityComponent = public class
  private
  protected
  public
    /// <summary>Calculates all biodiversity variables</summary>
    method Run(clock: Clock; state: IBioDiversityState);    
  end;
  
implementation


method BioDiversityComponent.Run(clock: Clock; state: IBioDiversityState); 
begin
  var s:=state;
  var t:=clock.Current;
  
  if t>Timestep.FromYear(2000) then
  begin
    var dt:=s.temp[t]-s.temp[t-1];
    
    s.nospecies[t]:=Math.Max(
      s.nospecbase/100,
      s.nospecies[t-1]*(1.0-s.bioloss-s.biosens*dt*dt/s.dbsta/s.dbsta)
      );    
  end
  else
    s.nospecies[t]:=s.nospecbase;
end;

end.