// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund.Components.ImpactHeating;

interface

uses Esmf,
     Fund.CommonDimensions;

type
  
  IImpactHeatingState = public interface
    property heating: IVariable2Dimensional<Timestep, Region, double> read;
    
    property hebm: IParameter1Dimensional<Region, double> read;
    property gdp90: IParameter1Dimensional<Region, double> read;
    
    property population: IParameter2Dimensional<Timestep, Region, double> read;
    property pop90: IParameter1Dimensional<Region, double> read;
    
    property income: IParameter2Dimensional<Timestep, Region, double> read;
    property heel: Double read;
    
    property temp: IParameter1Dimensional<Timestep, double> read;
    property henl: Double read;
        
    property cumaeei: IParameter2Dimensional<Timestep, Region, double> read;    
  end;
  
  ImpactHeatingComponent = public class
  private
  protected
  public
    method Run(clock: Clock; state: IImpactHeatingState);        
  end;
  
implementation

method ImpactHeatingComponent.Run(clock: Clock; state: IImpactHeatingState); 
begin
  var s:=state;
  var t:=clock.Current;

  for each r: Region in Enum.GetValues(typeof(Region)) do
  begin  
    var ypc: Double:=s.income[t,r]/s.population[t,r]*1000.0;
    var ypc90: Double:=s.gdp90[r]/s.pop90[r]*1000.0;
    
    s.heating[t,r]:=s.hebm[r]*s.cumaeei[t,r]*s.gdp90[r]*Math.Pow(s.temp[t]/1.0,s.henl)*Math.Pow(ypc/ypc90,s.heel)*s.population[t,r]/s.pop90[r];              
  end;  
end;

end.