// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

uses
  System.Configuration,
  System.IO;

type
  TCEARun = class
  private
    { Private Declarations }
    m_GlobalInputCsv: TextWriter;
    m_RegionInputCsv: TextWriter;
    m_YearInputCsv: TextWriter;
    m_RegionYearInputCsv: TextWriter;    
    
    _run: Run;
    _parameters: FundParameter;
        
    procedure Run(tax: double; VAR brf, erf: double);
    procedure CEARun(Output: ModelOutput; tax: Double; out maxrf: double);
    procedure DoOneRun(RunId: Integer; tax: Double; out maxrf: double);
       
  public
    constructor(runData: Run; parameters: FundParameter);

    procedure CEA;

    property GlobalInputCsv: TextWriter read m_GlobalInputCsv write m_GlobalInputCsv;
    property RegionInputCsv: TextWriter read m_RegionInputCsv write m_RegionInputCsv;
    property YearInputCsv: TextWriter read m_YearInputCsv write m_YearInputCsv;
    property RegionYearInputCsv: TextWriter read m_RegionYearInputCsv write m_RegionYearInputCsv;    
  end;

implementation

constructor TCEARun(runData: Run; parameters: FundParameter);
begin
  inherited constructor();

  _run:=runData;
  _parameters:=parameters;
end;

procedure TCEARun.CEA;

var tax, brf, erf, target: double;

begin
   tax:= 0.0;
   target:= 6.0;
   run(tax, var brf, var erf);
   Console.WriteLine(brf);
   IF _run.MonteCarloRuns = 0 THEN BEGIN
      WHILE (brf > target) AND (tax < 100) DO BEGIN
          tax:= tax + 1;
          Console.WriteLine(tax);
          run(tax,var brf,var erf);
          Console.Writeline(brf)
      END;
      IF tax > 0 THEN
         tax:= tax - 1;
      run(tax,var brf,var erf);
      WHILE (brf > target) AND (tax < 100) DO BEGIN
          tax:= tax + 0.1;
          Console.WriteLine(tax);
          run(tax,var brf,var erf);
          Console.Writeline(brf)
      end
   END
   ELSE
       WHILE erf > target DO BEGIN
           tax:= tax + 0.1;
           run(tax,var brf,var erf)
       end;
   Console.WriteLine(tax);
   tax:= 0
end;

procedure TCEARun.Run(tax: double; VAR brf, erf: double);
var
  i: Integer;
  
begin
  erf:=0;
  _parameters.SetToBestGuessValue;
      
  for i:=0 TO _run.MonteCarloRuns DO
  begin
    if i=0 then
    begin
      _parameters.SetToBestGuessValue;
      _parameters.WriteValuesToCsv(_run.Scenario.Id, i,
           m_GlobalInputCsv, m_RegionInputCsv, m_YearInputCsv, m_RegionYearInputCsv);      
    end
    else
    begin
      try
        Console.SetCursorPosition(0, Console.CursorTop - 1);
      except
        on E: IOException do
        begin          
        end;
      end;
      Console.WriteLine('Run {0}                ', i.ToString);

      _parameters.SetToRandomValue;
      _parameters.WriteValuesToCsv(_run.Scenario.Id, i,
           m_GlobalInputCsv, m_RegionInputCsv, m_YearInputCsv, m_RegionYearInputCsv);      
    end;
    
    var maxrf:double:=0.0;
    
    DoOneRun(i, tax, out maxrf);
    if i = 0 then
       brf:= maxrf;
           
    erf:=erf + (maxrf/_run.MonteCarloRuns);
  end;
end;

procedure TCEARun.DoOneRun(RunId: Integer; tax: double; out maxrf: double);
var
  i_output: ModelOutput;
begin
  i_output := new ModelOutput;
  CEARun(i_output, tax, out maxrf);
end;

procedure TCEARun.CEARun(Output: ModelOutput; tax: double; out maxrf: double);
var
  fundWorkflow: FundWorkflow;
begin
  fundWorkflow:=new FundWorkflow(_parameters);
   
  fundWorkflow.ModelFields.SetNonDimensionalFieldValue('globalhotellingtaxation','basetax', tax);
  
  fundWorkflow.Run(Output);  
  
  maxrf:=fundWorkflow.ClimateState.maxrf;            
end;

end.
