// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

uses Esmf;

type
  
  ISocioEconomic2Climate = public interface

    /// <summary>co2 emissions, maybe?</summary>
    property mco2: IVariable1Dimensional<Timestep, double> read;
            
    /// <summary>Global CH4 emissions</summary>
    property globch4: IVariable1Dimensional<Timestep, double> read;
    
    /// <summary>Global N2O emissions</summary>
    property globn2o: IVariable1Dimensional<Timestep, double> read;
    
    /// <summary> Global SF6 emissions</summary>
    property globsf6: IVariable1Dimensional<Timestep, double> read;
  
    /// <summary>Global SO2 emissions</summary>
    property globso2: IVariable1Dimensional<Timestep, double> read;
    
    
    property emissionwithforestry: IParameter2Dimensional<Timestep, Fund.CommonDimensions.Region, double> read;
    property ch4: IParameter2Dimensional<Timestep, Fund.CommonDimensions.Region, double> read;
    property n2o: IParameter2Dimensional<Timestep, Fund.CommonDimensions.Region, double> read;
    property sf6: IParameter2Dimensional<Timestep, Fund.CommonDimensions.Region, double> read;
    property so2withseeiandscei: IParameter2Dimensional<Timestep, Fund.CommonDimensions.Region, double> read;
  end; 
   
  SocioEconomic2Climate = public class
  private
  protected
  public
    method Run(clock: Clock; state: ISocioEconomic2Climate);
  end;
  
implementation

method SocioEconomic2Climate.Run(clock: Clock; state: ISocioEconomic2Climate); 
var
 globco2: double;
 globch4: double;
 globn2o: double;
 globsf6: double;
 globso2: double;
begin
  var s:=state;
  var t:=clock.Current;
  // Calculate global co2, ch4, n2o, sf6 and so2 emissions    
  globco2:= 0;
  globch4:= 0;
  globn2o:= 0;
  globsf6:= 0;
  globso2:= 34.4;
  
  for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
  begin
    globco2:=globco2 + s.emissionwithforestry[t,r];
    globch4:= globch4 + s.ch4[t,r];
    globn2o:= globn2o + s.n2o[t,r];
    globsf6:= globsf6 + s.sf6[t,r];
    globso2:= globso2 + s.so2withseeiandscei[t,r];
  end;

  s.mco2[t]:=globco2;
  s.globch4[t]:=globch4;
  s.globn2o[t]:=globn2o;
  s.globsf6[t]:=globsf6;
  s.globso2[t]:=globso2;
    
end;

end.