// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

uses System.Collections.Generic,
     System.Reflection.Emit,
     Esmf;

type
  RegionParameterConnector<D1> = public class (Esmf.IParameter1Dimensional<D1,double>)
  private
    _values: array of Double;
    _enumConversionDelegate: Enum2IntConversionDelegate<D1>;
  protected
    method GetItem(d: D1): Double;
  public
    constructor(param: Parameters; name: string);
    
    property Item[d: D1]: double read GetItem; default;
    method GetEnumerator: System.Collections.Generic.IEnumerable<Esmf.Parameter1DimensionalMember<D1,System.Double>>;
  end;
   
implementation

method RegionParameterConnector<d1>.GetItem(d: D1): Double; 
begin
  Result:=_values[_enumConversionDelegate(d)-1];    
end;

constructor RegionParameterConnector<d1>(param: Parameters; name: string); 
begin 
  if typeof(D1)=typeof(Esmf.Timestep) then
  begin
    raise new InvalidOperationException();
  end  
  else if typeof(D1).BaseType=typeof(System.Enum) then
  begin
    var values := array of Integer(Enum.GetValues(typeof(D1)));        
       
    _values:=new Double[16];
    for each r: Integer in values do
    begin
      _values[r-1]:=param.GetItemByRegion(Enum.GetName(typeof(D1),r),name);
    end; 
    
    _enumConversionDelegate:=Enum2IntConversionSingleton<D1>.Instance.Delegate;
  end
  else
    raise new InvalidOperationException();
  
end;

method RegionParameterConnector<D1>.GetEnumerator: System.Collections.Generic.IEnumerable<Esmf.Parameter1DimensionalMember<D1,System.Double>>;
begin
  raise new NotImplementedException;
end;

end.