// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

uses System.Collections.Generic;

type
  
  Fund28LegacyWeightingCombinations = public static class
  private
    procedure AdaWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
    procedure DavidWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
    procedure DefraWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
    procedure PaulWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
    procedure JiehansWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
    procedure StandardWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
    procedure CameronWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
    procedure MeganWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
    procedure DritanWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
    procedure NeedsWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
    procedure EpaWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
    procedure RawWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
    procedure RhoEtaSensitivityWeightingCombintaiontsPart1(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
    procedure RhoEtaSensitivityWeightingCombintaiontsPart2(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
    procedure RhoEtaSensitivityWeightingCombintaiontsPart3(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
    procedure RhoEtaSensitivityWeightingCombintaiontsPart4(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);    
  public
    method GetWeightingCombinationsFromName(name: String; out i_WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
  end;

implementation

procedure Fund28LegacyWeightingCombinations.CameronWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
var
  i_weightingCombinations: array [0..0] of WeightingCombination;
begin
//  i_weightingCombinations[0]:=WeightingCombination.Create;
//  i_weightingCombinations[0].Add(ConstantDiscounting.Create(0.00));
//
//  i_weightingCombinations[1]:=WeightingCombination.Create;
//  i_weightingCombinations[1].Add(ConstantDiscounting.Create(0.01));
//
//  i_weightingCombinations[2]:=WeightingCombination.Create;
//  i_weightingCombinations[2].Add(ConstantDiscounting.Create(0.03));
//
//  i_weightingCombinations[3]:=WeightingCombination.Create;
//  i_weightingCombinations[3].Add(ConstantDiscountRate.Create(0.05));
//
//  i_weightingCombinations[4]:=WeightingCombination.Create;
//  i_weightingCombinations[4].Add(ConstantDiscountRate.Create(0.07));
//
//  i_weightingCombinations[5]:=WeightingCombination.Create;
//  i_weightingCombinations[5].Add(ConstantDiscountRate.Create(0.10));
//
//  i_weightingCombinations[6]:=WeightingCombination.Create;
//  i_weightingCombinations[6].Add(ConstantDiscountRate.Create(0.12));
  
  i_weightingCombinations[0]:=new WeightingCombination;
  i_weightingCombinations[0].Add(new UnityWeight);

  WeightingCombinations:=i_weightingCombinations;
end;

procedure Fund28LegacyWeightingCombinations.JiehansWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
var
  i_weightingCombinations: array [0..61] of WeightingCombination;
begin
  i_weightingCombinations[0]:=new WeightingCombination;
  i_weightingCombinations[0].Add(new ConstantDiscounting(0.00));

  i_weightingCombinations[1]:=new WeightingCombination;
  i_weightingCombinations[1].Add(new ConstantDiscounting(0.00));
  i_weightingCombinations[1].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[2]:=new WeightingCombination;
  i_weightingCombinations[2].Add(new ConstantDiscounting(0.01));

  i_weightingCombinations[3]:=new WeightingCombination;
  i_weightingCombinations[3].Add(new ConstantDiscounting(0.01));
  i_weightingCombinations[3].Add(new AverageEquityWeightingFirstPeriod);


  i_weightingCombinations[4]:=new WeightingCombination;
  i_weightingCombinations[4].Add(new ConstantDiscounting(0.03));

  i_weightingCombinations[5]:=new WeightingCombination;
  i_weightingCombinations[5].Add(new ConstantDiscounting(0.03));
  i_weightingCombinations[5].Add(new AverageEquityWeightingFirstPeriod);


  i_weightingCombinations[6]:=new WeightingCombination;
  i_weightingCombinations[6].Add(new GreenbookDiscounting(50));

  i_weightingCombinations[7]:=new WeightingCombination;
  i_weightingCombinations[7].Add(new ConstantDiscountRate(0.035, 'prtp'));

  i_weightingCombinations[8]:=new WeightingCombination;
  i_weightingCombinations[8].Add(new ConstantDiscountRate(0.035, 'prtp'));
  i_weightingCombinations[8].Add(new AverageEquityWeighting);

  i_weightingCombinations[9]:=new WeightingCombination;
  i_weightingCombinations[9].Add(new WeitzmanStepDecline);

  i_weightingCombinations[10]:=new WeightingCombination;
  i_weightingCombinations[10].Add(new WeitzmanStepDecline);
  i_weightingCombinations[10].Add(new AverageEquityWeighting);

  i_weightingCombinations[11]:=new WeightingCombination;
  i_weightingCombinations[11].Add(new ConstantDiscountRate(0.04, 'prtp'));

  i_weightingCombinations[12]:=new WeightingCombination;
  i_weightingCombinations[12].Add(new ConstantDiscountRate(0.04, 'prtp'));
  i_weightingCombinations[12].Add(new AverageEquityWeighting);

  i_weightingCombinations[13]:=new WeightingCombination;
  i_weightingCombinations[13].Add(new WeitzmanDiscounting(0.04, 0.03));

  i_weightingCombinations[14]:=new WeightingCombination;
  i_weightingCombinations[14].Add(new WeitzmanDiscounting(0.04, 0.03));
  i_weightingCombinations[14].Add(new AverageEquityWeighting);

  i_weightingCombinations[15]:=new WeightingCombination;
  i_weightingCombinations[15].Add(new GollierUncertainGrowthDiscounting(0.00, 0.01, 0.01));

  i_weightingCombinations[16]:=new WeightingCombination;
  i_weightingCombinations[16].Add(new GollierUncertainGrowthDiscounting(0.00, 0.01, 0.01));
  i_weightingCombinations[16].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[17]:=new WeightingCombination;
  i_weightingCombinations[17].Add(new GollierUncertainGrowthDiscounting(0.00, 0.03, 0.03));

  i_weightingCombinations[18]:=new WeightingCombination;
  i_weightingCombinations[18].Add(new GollierUncertainGrowthDiscounting(0.00, 0.03, 0.03));
  i_weightingCombinations[18].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[19]:=new WeightingCombination;
  i_weightingCombinations[19].Add(new GollierUncertainGrowthDiscounting(0.00, 0.05, 0.05));

  i_weightingCombinations[20]:=new WeightingCombination;
  i_weightingCombinations[20].Add(new GollierUncertainGrowthDiscounting(0.00, 0.05, 0.05));
  i_weightingCombinations[20].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[21]:=new WeightingCombination;
  i_weightingCombinations[21].Add(new GollierUncertainGrowthDiscounting(0.01, 0.01, 0.01));

  i_weightingCombinations[22]:=new WeightingCombination;
  i_weightingCombinations[22].Add(new GollierUncertainGrowthDiscounting(0.01, 0.01, 0.01));
  i_weightingCombinations[22].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[23]:=new WeightingCombination;
  i_weightingCombinations[23].Add(new GollierUncertainGrowthDiscounting(0.01, 0.03, 0.03));

  i_weightingCombinations[24]:=new WeightingCombination;
  i_weightingCombinations[24].Add(new GollierUncertainGrowthDiscounting(0.01, 0.03, 0.03));
  i_weightingCombinations[24].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[25]:=new WeightingCombination;
  i_weightingCombinations[25].Add(new GollierUncertainGrowthDiscounting(0.01, 0.05, 0.05));

  i_weightingCombinations[26]:=new WeightingCombination;
  i_weightingCombinations[26].Add(new GollierUncertainGrowthDiscounting(0.01, 0.05, 0.05));
  i_weightingCombinations[26].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[27]:=new WeightingCombination;
  i_weightingCombinations[27].Add(new GollierUncertainGrowthDiscounting(0.03, 0.01, 0.01));

  i_weightingCombinations[28]:=new WeightingCombination;
  i_weightingCombinations[28].Add(new GollierUncertainGrowthDiscounting(0.03, 0.01, 0.01));
  i_weightingCombinations[28].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[29]:=new WeightingCombination;
  i_weightingCombinations[29].Add(new GollierUncertainGrowthDiscounting(0.03, 0.03, 0.03));

  i_weightingCombinations[30]:=new WeightingCombination;
  i_weightingCombinations[30].Add(new GollierUncertainGrowthDiscounting(0.03, 0.03, 0.03));
  i_weightingCombinations[30].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[31]:=new WeightingCombination;
  i_weightingCombinations[31].Add(new GollierUncertainGrowthDiscounting(0.03, 0.05, 0.05));

  i_weightingCombinations[32]:=new WeightingCombination;
  i_weightingCombinations[32].Add(new GollierUncertainGrowthDiscounting(0.03, 0.05, 0.05));
  i_weightingCombinations[32].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[33]:=new WeightingCombination;
  i_weightingCombinations[33].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.00, 0.005));

  i_weightingCombinations[34]:=new WeightingCombination;
  i_weightingCombinations[34].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.00, 0.005));
  i_weightingCombinations[34].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[35]:=new WeightingCombination;
  i_weightingCombinations[35].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.00, 0.01));

  i_weightingCombinations[36]:=new WeightingCombination;
  i_weightingCombinations[36].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.00, 0.01));
  i_weightingCombinations[36].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[37]:=new WeightingCombination;
  i_weightingCombinations[37].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.00, 0.015));

  i_weightingCombinations[38]:=new WeightingCombination;
  i_weightingCombinations[38].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.00, 0.015));
  i_weightingCombinations[38].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[39]:=new WeightingCombination;
  i_weightingCombinations[39].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.01, 0.005));

  i_weightingCombinations[40]:=new WeightingCombination;
  i_weightingCombinations[40].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.01, 0.005));
  i_weightingCombinations[40].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[41]:=new WeightingCombination;
  i_weightingCombinations[41].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.01, 0.01));

  i_weightingCombinations[42]:=new WeightingCombination;
  i_weightingCombinations[42].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.01, 0.01));
  i_weightingCombinations[42].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[43]:=new WeightingCombination;
  i_weightingCombinations[43].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.01, 0.015));

  i_weightingCombinations[44]:=new WeightingCombination;
  i_weightingCombinations[44].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.01, 0.015));
  i_weightingCombinations[44].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[45]:=new WeightingCombination;
  i_weightingCombinations[45].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.03, 0.005));

  i_weightingCombinations[46]:=new WeightingCombination;
  i_weightingCombinations[46].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.03, 0.005));
  i_weightingCombinations[46].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[47]:=new WeightingCombination;
  i_weightingCombinations[47].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.03, 0.01));

  i_weightingCombinations[48]:=new WeightingCombination;
  i_weightingCombinations[48].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.03, 0.01));
  i_weightingCombinations[48].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[49]:=new WeightingCombination;
  i_weightingCombinations[49].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.03, 0.015));

  i_weightingCombinations[50]:=new WeightingCombination;
  i_weightingCombinations[50].Add(new GollierUncertainGrowthDiscountingNormalDistribution(0.03, 0.015));
  i_weightingCombinations[50].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[51]:=new WeightingCombination;
  i_weightingCombinations[51].Add(new HeterogeneousPRTP(0.064));

  i_weightingCombinations[52]:=new WeightingCombination;
  i_weightingCombinations[52].Add(new HeterogeneousPRTP(0.064));
  i_weightingCombinations[52].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[53]:=new WeightingCombination;
  i_weightingCombinations[53].Add(new HeterogeneousPRTP(0.07));

  i_weightingCombinations[54]:=new WeightingCombination;
  i_weightingCombinations[54].Add(new HeterogeneousPRTP(0.07));
  i_weightingCombinations[54].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[55]:=new WeightingCombination;
  i_weightingCombinations[55].Add(new HeterogeneousPRTP(0.03));

  i_weightingCombinations[56]:=new WeightingCombination;
  i_weightingCombinations[56].Add(new HeterogeneousPRTP(0.03));
  i_weightingCombinations[56].Add(new AverageEquityWeightingFirstPeriod);

  i_weightingCombinations[57]:=new WeightingCombination;
  i_weightingCombinations[57].Add(new GreenbookDiscounting(50));
  i_weightingCombinations[57].Add(new AverageEquityWeighting);

  i_weightingCombinations[58]:=new WeightingCombination;
  i_weightingCombinations[58].Add(new ConstantDiscountRate(0.064, 'prtp'));

  i_weightingCombinations[59]:=new WeightingCombination;
  i_weightingCombinations[59].Add(new ConstantDiscountRate(0.064, 'prtp'));
  i_weightingCombinations[59].Add(new AverageEquityWeighting);

  i_weightingCombinations[60]:=new WeightingCombination;
  i_weightingCombinations[60].Add(new ConstantDiscountRate(0.07, 'prtp'));

  i_weightingCombinations[61]:=new WeightingCombination;
  i_weightingCombinations[61].Add(new ConstantDiscountRate(0.07, 'prtp'));
  i_weightingCombinations[61].Add(new AverageEquityWeighting);

  WeightingCombinations:=i_weightingCombinations;
end;

procedure Fund28LegacyWeightingCombinations.StandardWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
var
  i_weightingCombinations: array [0..6] of WeightingCombination;
begin
  i_weightingCombinations[0]:=new WeightingCombination;
  i_weightingCombinations[0].Add(new ConstantDiscounting(0.00));

  i_weightingCombinations[1]:=new WeightingCombination;
  i_weightingCombinations[1].Add(new ConstantDiscounting(0.00));
  i_weightingCombinations[1].Add(new AverageEquityWeighting);

  i_weightingCombinations[2]:=new WeightingCombination;
  i_weightingCombinations[2].Add(new ConstantDiscounting(0.01));

  i_weightingCombinations[3]:=new WeightingCombination;
  i_weightingCombinations[3].Add(new ConstantDiscounting(0.01));
  i_weightingCombinations[3].Add(new AverageEquityWeighting);

  i_weightingCombinations[4]:=new WeightingCombination;
  i_weightingCombinations[4].Add(new ConstantDiscounting(0.03));

  i_weightingCombinations[5]:=new WeightingCombination;
  i_weightingCombinations[5].Add(new ConstantDiscounting(0.03));
  i_weightingCombinations[5].Add(new AverageEquityWeighting);

  i_weightingCombinations[6]:=new WeightingCombination;
  i_weightingCombinations[6].Add(new GreenbookDiscounting(50));

  WeightingCombinations:=i_weightingCombinations;
end;

procedure Fund28LegacyWeightingCombinations.DefraWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
var
  i_weightingCombinations: array [0..8] of WeightingCombination;
begin
  i_weightingCombinations[0]:=new WeightingCombination;
  i_weightingCombinations[0].Add(new GreenbookDiscounting(Convert.ToInt32(parameters.emissionPeriod) - 1950));
  i_weightingCombinations[0].Add(new AverageEquityWeighting);

  i_weightingCombinations[1]:=new WeightingCombination;
  i_weightingCombinations[1].Add(new GreenbookDiscounting(Convert.ToInt32(parameters.emissionPeriod) - 1950));

  i_weightingCombinations[2]:=new WeightingCombination;
  i_weightingCombinations[2].Add(new AnthoffEquityWeighting(50, 'WEU', 1));

  i_weightingCombinations[3]:=new WeightingCombination;
  i_weightingCombinations[3].Add(new ConstantDiscounting(0.00));

  i_weightingCombinations[4]:=new WeightingCombination;
  i_weightingCombinations[4].Add(new AnthoffEquityWeighting(50, 'WEU', 1));
  i_weightingCombinations[4].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[5]:=new WeightingCombination;
  i_weightingCombinations[5].Add(new ConstantDiscounting(0.01));

  i_weightingCombinations[6]:=new WeightingCombination;
  i_weightingCombinations[6].Add(new AnthoffEquityWeighting(50, 'WEU', 1));
  i_weightingCombinations[6].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[7]:=new WeightingCombination;
  i_weightingCombinations[7].Add(new ConstantDiscounting(0.03));

  i_weightingCombinations[8]:=new WeightingCombination;


  WeightingCombinations:=i_weightingCombinations;
end;

procedure Fund28LegacyWeightingCombinations.PaulWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
var
  i_weightingCombinations: array [0..1] of WeightingCombination;
begin
  i_weightingCombinations[0]:=new WeightingCombination;
  i_weightingCombinations[0].Add(new GreenbookDiscounting(Convert.ToInt32(parameters.emissionPeriod) - 1950));
  i_weightingCombinations[0].Add(new AverageEquityWeighting);

  i_weightingCombinations[1]:=new WeightingCombination;
  i_weightingCombinations[1].Add(new GreenbookDiscounting(Convert.ToInt32(parameters.emissionPeriod) - 1950));

  WeightingCombinations:=i_weightingCombinations;
end;


procedure Fund28LegacyWeightingCombinations.AdaWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
var
  i_weightingCombinations: array [0..3] of WeightingCombination;
begin
  i_weightingCombinations[0]:=new WeightingCombination;
  i_weightingCombinations[0].Add(new ConstantDiscounting(0.00));

  i_weightingCombinations[1]:=new WeightingCombination;
  i_weightingCombinations[1].Add(new ConstantDiscounting(0.01));

  i_weightingCombinations[2]:=new WeightingCombination;
  i_weightingCombinations[2].Add(new ConstantDiscounting(0.03));

  i_weightingCombinations[3]:=new WeightingCombination;
  i_weightingCombinations[3].Add(new GreenbookDiscounting(50));

  WeightingCombinations:=i_weightingCombinations;
end;

procedure Fund28LegacyWeightingCombinations.DavidWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
var
  i_weightingCombinations: array [0..65] of WeightingCombination;
begin
  i_weightingCombinations[0]:=new WeightingCombination;
  i_weightingCombinations[0].Add(new ConstantDiscounting(0.00));

  i_weightingCombinations[1]:=new WeightingCombination;
  i_weightingCombinations[1].Add(new ConstantDiscounting(0.01));

  i_weightingCombinations[2]:=new WeightingCombination;
  i_weightingCombinations[2].Add(new ConstantDiscounting(0.03));

  i_weightingCombinations[3]:=new WeightingCombination;
  i_weightingCombinations[3].Add(new AnthoffEquityWeighting(50, '', 1));
  i_weightingCombinations[3].Add(new ConstantDiscountRate(0.00, 'prtp'));

  i_weightingCombinations[4]:=new WeightingCombination;
  i_weightingCombinations[4].Add(new AnthoffEquityWeighting(50, '', 1));
  i_weightingCombinations[4].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[5]:=new WeightingCombination;
  i_weightingCombinations[5].Add(new AnthoffEquityWeighting(50, '', 1));
  i_weightingCombinations[5].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[6]:=new WeightingCombination;
  i_weightingCombinations[6].Add(new AnthoffEquityWeighting(50, 'USA', 1));
  i_weightingCombinations[6].Add(new ConstantDiscountRate(0.00, 'prtp'));

  i_weightingCombinations[7]:=new WeightingCombination;
  i_weightingCombinations[7].Add(new AnthoffEquityWeighting(50, 'USA', 1));
  i_weightingCombinations[7].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[8]:=new WeightingCombination;
  i_weightingCombinations[8].Add(new AnthoffEquityWeighting(50, 'USA', 1));
  i_weightingCombinations[8].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[9]:=new WeightingCombination;
  i_weightingCombinations[9].Add(new AnthoffEquityWeighting(50, 'USA', 1));
  i_weightingCombinations[9].Add(new ConstantDiscountRate(0.00, 'prtp'));
  i_weightingCombinations[9].Add(new AnthoffAggregationFactor(parameters));

  i_weightingCombinations[10]:=new WeightingCombination;
  i_weightingCombinations[10].Add(new AnthoffEquityWeighting(50, 'USA', 1));
  i_weightingCombinations[10].Add(new ConstantDiscountRate(0.01, 'prtp'));
  i_weightingCombinations[10].Add(new AnthoffAggregationFactor(parameters));

  i_weightingCombinations[11]:=new WeightingCombination;
  i_weightingCombinations[11].Add(new AnthoffEquityWeighting(50, 'USA', 1));
  i_weightingCombinations[11].Add(new ConstantDiscountRate(0.03, 'prtp'));
  i_weightingCombinations[11].Add(new AnthoffAggregationFactor(parameters));

  i_weightingCombinations[12]:=new WeightingCombination;
  i_weightingCombinations[12].Add(new AnthoffEquityWeighting(50, 'USA', 0.5));
  i_weightingCombinations[12].Add(new ConstantDiscountRate(0.00, 'prtp'));

  i_weightingCombinations[13]:=new WeightingCombination;
  i_weightingCombinations[13].Add(new AnthoffEquityWeighting(50, 'USA', 0.5));
  i_weightingCombinations[13].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[14]:=new WeightingCombination;
  i_weightingCombinations[14].Add(new AnthoffEquityWeighting(50, 'USA', 0.5));
  i_weightingCombinations[14].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[15]:=new WeightingCombination;
  i_weightingCombinations[15].Add(new AnthoffEquityWeighting(50, 'USA', 1.2));
  i_weightingCombinations[15].Add(new ConstantDiscountRate(0.00, 'prtp'));

  i_weightingCombinations[16]:=new WeightingCombination;
  i_weightingCombinations[16].Add(new AnthoffEquityWeighting(50, 'USA', 1.2));
  i_weightingCombinations[16].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[17]:=new WeightingCombination;
  i_weightingCombinations[17].Add(new AnthoffEquityWeighting(50, 'USA', 1.2));
  i_weightingCombinations[17].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[18]:=new WeightingCombination;
  i_weightingCombinations[18].Add(new AnthoffEquityWeighting(50, 'CAN', 1));
  i_weightingCombinations[18].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[19]:=new WeightingCombination;
  i_weightingCombinations[19].Add(new AnthoffEquityWeighting(50, 'WEU', 1));
  i_weightingCombinations[19].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[20]:=new WeightingCombination;
  i_weightingCombinations[20].Add(new AnthoffEquityWeighting(50, 'JPK', 1));
  i_weightingCombinations[20].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[21]:=new WeightingCombination;
  i_weightingCombinations[21].Add(new AnthoffEquityWeighting(50, 'ANZ', 1));
  i_weightingCombinations[21].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[22]:=new WeightingCombination;
  i_weightingCombinations[22].Add(new AnthoffEquityWeighting(50, 'EEU', 1));
  i_weightingCombinations[22].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[23]:=new WeightingCombination;
  i_weightingCombinations[23].Add(new AnthoffEquityWeighting(50, 'FSU', 1));
  i_weightingCombinations[23].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[24]:=new WeightingCombination;
  i_weightingCombinations[24].Add(new AnthoffEquityWeighting(50, 'MDE', 1));
  i_weightingCombinations[24].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[25]:=new WeightingCombination;
  i_weightingCombinations[25].Add(new AnthoffEquityWeighting(50, 'CAM', 1));
  i_weightingCombinations[25].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[26]:=new WeightingCombination;
  i_weightingCombinations[26].Add(new AnthoffEquityWeighting(50, 'LAM', 1));
  i_weightingCombinations[26].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[27]:=new WeightingCombination;
  i_weightingCombinations[27].Add(new AnthoffEquityWeighting(50, 'SAS', 1));
  i_weightingCombinations[27].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[28]:=new WeightingCombination;
  i_weightingCombinations[28].Add(new AnthoffEquityWeighting(50, 'SEA', 1));
  i_weightingCombinations[28].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[29]:=new WeightingCombination;
  i_weightingCombinations[29].Add(new AnthoffEquityWeighting(50, 'CHI', 1));
  i_weightingCombinations[29].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[30]:=new WeightingCombination;
  i_weightingCombinations[30].Add(new AnthoffEquityWeighting(50, 'NAF', 1));
  i_weightingCombinations[30].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[31]:=new WeightingCombination;
  i_weightingCombinations[31].Add(new AnthoffEquityWeighting(50, 'SSA', 1));
  i_weightingCombinations[31].Add(new ConstantDiscountRate(0.01, 'prtp'));

  i_weightingCombinations[32]:=new WeightingCombination;
  i_weightingCombinations[32].Add(new AnthoffEquityWeighting(50, 'SIS', 1));
  i_weightingCombinations[32].Add(new ConstantDiscountRate(0.01, 'prtp'));
                              
  //-
  
  i_weightingCombinations[33]:=new WeightingCombination;
  i_weightingCombinations[33].Add(new AnthoffEquityWeighting(50, 'CAN', 1));
  i_weightingCombinations[33].Add(new ConstantDiscountRate(0, 'prtp'));

  i_weightingCombinations[34]:=new WeightingCombination;
  i_weightingCombinations[34].Add(new AnthoffEquityWeighting(50, 'WEU', 1));
  i_weightingCombinations[34].Add(new ConstantDiscountRate(0, 'prtp'));

  i_weightingCombinations[35]:=new WeightingCombination;
  i_weightingCombinations[35].Add(new AnthoffEquityWeighting(50, 'JPK', 1));
  i_weightingCombinations[35].Add(new ConstantDiscountRate(0, 'prtp'));

  i_weightingCombinations[36]:=new WeightingCombination;
  i_weightingCombinations[36].Add(new AnthoffEquityWeighting(50, 'ANZ', 1));
  i_weightingCombinations[36].Add(new ConstantDiscountRate(0, 'prtp'));

  i_weightingCombinations[37]:=new WeightingCombination;
  i_weightingCombinations[37].Add(new AnthoffEquityWeighting(50, 'EEU', 1));
  i_weightingCombinations[37].Add(new ConstantDiscountRate(0, 'prtp'));

  i_weightingCombinations[38]:=new WeightingCombination;
  i_weightingCombinations[38].Add(new AnthoffEquityWeighting(50, 'FSU', 1));
  i_weightingCombinations[38].Add(new ConstantDiscountRate(0, 'prtp'));

  i_weightingCombinations[39]:=new WeightingCombination;
  i_weightingCombinations[39].Add(new AnthoffEquityWeighting(50, 'MDE', 1));
  i_weightingCombinations[39].Add(new ConstantDiscountRate(0, 'prtp'));

  i_weightingCombinations[40]:=new WeightingCombination;
  i_weightingCombinations[40].Add(new AnthoffEquityWeighting(50, 'CAM', 1));
  i_weightingCombinations[40].Add(new ConstantDiscountRate(0, 'prtp'));

  i_weightingCombinations[41]:=new WeightingCombination;
  i_weightingCombinations[41].Add(new AnthoffEquityWeighting(50, 'LAM', 1));
  i_weightingCombinations[41].Add(new ConstantDiscountRate(0, 'prtp'));

  i_weightingCombinations[42]:=new WeightingCombination;
  i_weightingCombinations[42].Add(new AnthoffEquityWeighting(50, 'SAS', 1));
  i_weightingCombinations[42].Add(new ConstantDiscountRate(0, 'prtp'));

  i_weightingCombinations[43]:=new WeightingCombination;
  i_weightingCombinations[43].Add(new AnthoffEquityWeighting(50, 'SEA', 1));
  i_weightingCombinations[43].Add(new ConstantDiscountRate(0, 'prtp'));

  i_weightingCombinations[44]:=new WeightingCombination;
  i_weightingCombinations[44].Add(new AnthoffEquityWeighting(50, 'CHI', 1));
  i_weightingCombinations[44].Add(new ConstantDiscountRate(0, 'prtp'));

  i_weightingCombinations[45]:=new WeightingCombination;
  i_weightingCombinations[45].Add(new AnthoffEquityWeighting(50, 'NAF', 1));
  i_weightingCombinations[45].Add(new ConstantDiscountRate(0, 'prtp'));

  i_weightingCombinations[46]:=new WeightingCombination;
  i_weightingCombinations[46].Add(new AnthoffEquityWeighting(50, 'SSA', 1));
  i_weightingCombinations[46].Add(new ConstantDiscountRate(0, 'prtp'));

  i_weightingCombinations[47]:=new WeightingCombination;
  i_weightingCombinations[47].Add(new AnthoffEquityWeighting(50, 'SIS', 1));
  i_weightingCombinations[47].Add(new ConstantDiscountRate(0, 'prtp'));
  
  //--
  
  i_weightingCombinations[48]:=new WeightingCombination;
  i_weightingCombinations[48].Add(new AnthoffEquityWeighting(50, 'CAN', 1));
  i_weightingCombinations[48].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[49]:=new WeightingCombination;
  i_weightingCombinations[49].Add(new AnthoffEquityWeighting(50, 'WEU', 1));
  i_weightingCombinations[49].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[50]:=new WeightingCombination;
  i_weightingCombinations[50].Add(new AnthoffEquityWeighting(50, 'JPK', 1));
  i_weightingCombinations[50].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[51]:=new WeightingCombination;
  i_weightingCombinations[51].Add(new AnthoffEquityWeighting(50, 'ANZ', 1));
  i_weightingCombinations[51].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[52]:=new WeightingCombination;
  i_weightingCombinations[52].Add(new AnthoffEquityWeighting(50, 'EEU', 1));
  i_weightingCombinations[52].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[53]:=new WeightingCombination;
  i_weightingCombinations[53].Add(new AnthoffEquityWeighting(50, 'FSU', 1));
  i_weightingCombinations[53].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[54]:=new WeightingCombination;
  i_weightingCombinations[54].Add(new AnthoffEquityWeighting(50, 'MDE', 1));
  i_weightingCombinations[54].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[55]:=new WeightingCombination;
  i_weightingCombinations[55].Add(new AnthoffEquityWeighting(50, 'CAM', 1));
  i_weightingCombinations[55].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[56]:=new WeightingCombination;
  i_weightingCombinations[56].Add(new AnthoffEquityWeighting(50, 'LAM', 1));
  i_weightingCombinations[56].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[57]:=new WeightingCombination;
  i_weightingCombinations[57].Add(new AnthoffEquityWeighting(50, 'SAS', 1));
  i_weightingCombinations[57].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[58]:=new WeightingCombination;
  i_weightingCombinations[58].Add(new AnthoffEquityWeighting(50, 'SEA', 1));
  i_weightingCombinations[58].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[59]:=new WeightingCombination;
  i_weightingCombinations[59].Add(new AnthoffEquityWeighting(50, 'CHI', 1));
  i_weightingCombinations[59].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[60]:=new WeightingCombination;
  i_weightingCombinations[60].Add(new AnthoffEquityWeighting(50, 'NAF', 1));
  i_weightingCombinations[60].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[61]:=new WeightingCombination;
  i_weightingCombinations[61].Add(new AnthoffEquityWeighting(50, 'SSA', 1));
  i_weightingCombinations[61].Add(new ConstantDiscountRate(0.03, 'prtp'));

  i_weightingCombinations[62]:=new WeightingCombination;
  i_weightingCombinations[62].Add(new AnthoffEquityWeighting(50, 'SIS', 1));
  i_weightingCombinations[62].Add(new ConstantDiscountRate(0.03, 'prtp')); 
  
  // -- Old Tol (1999) method
  i_weightingCombinations[63]:=new WeightingCombination;
  i_weightingCombinations[63].Add(new ConstantDiscounting(0.00));
  i_weightingCombinations[63].Add(new AverageEquityWeighting());    
     
  i_weightingCombinations[64]:=new WeightingCombination;
  i_weightingCombinations[64].Add(new ConstantDiscounting(0.01));
  i_weightingCombinations[64].Add(new AverageEquityWeighting());    

  i_weightingCombinations[65]:=new WeightingCombination;
  i_weightingCombinations[65].Add(new ConstantDiscounting(0.03));
  i_weightingCombinations[65].Add(new AverageEquityWeighting());    
    
  WeightingCombinations:=i_weightingCombinations;
end;

procedure Fund28LegacyWeightingCombinations.MeganWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
var
  i_weightingCombinations: array [0..3] of WeightingCombination;
begin
  i_weightingCombinations[0]:=new WeightingCombination;
  i_weightingCombinations[0].Add(new ConstantDiscounting(0.00));

  i_weightingCombinations[1]:=new WeightingCombination;
  i_weightingCombinations[1].Add(new ConstantDiscounting(0.01));

  i_weightingCombinations[2]:=new WeightingCombination;
  i_weightingCombinations[2].Add(new ConstantDiscounting(0.03));

  i_weightingCombinations[3]:=new WeightingCombination;
  i_weightingCombinations[3].Add(new GreenbookDiscounting(50));

  WeightingCombinations:=i_weightingCombinations;
end;

procedure Fund28LegacyWeightingCombinations.DritanWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
var
  i_weightingCombinations: array [0..0] of WeightingCombination;
begin
  i_weightingCombinations[0]:=new WeightingCombination;
  i_weightingCombinations[0].Add(new ConstantDiscounting(0.03));

  WeightingCombinations:=i_weightingCombinations;
end;

procedure Fund28LegacyWeightingCombinations.NeedsWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
var
  i_weightingCombinations:=new List<WeightingCombination>;
begin
  var prtps: array of double:=[0.0, 0.01, 0.03];
     
  var w: WeightingCombination;
    
  for each prtp: Double in prtps do
  begin      
    w:=new WeightingCombination(String.Format('{0}%:NoEW:Base2005', prtp*100.0));      
    w.Add(new ConstantDiscounting(prtp,55));
    i_weightingCombinations.Add(w);
    
    w:=new WeightingCombination(String.Format('{0}%:AvEW:Base2005', prtp*100.0));      
    w.Add(new AnthoffEquityWeighting(55, '', 1.0));
    w.Add(new ConstantDiscountRate(prtp, 55));
    i_weightingCombinations.Add(w);
    
    w:=new WeightingCombination(String.Format('{0}%:WeuEW:Base2005', prtp*100.0));      
    w.Add(new AnthoffEquityWeighting(55, 'WEU', 1.0));
    w.Add(new ConstantDiscountRate(prtp, 55));
    i_weightingCombinations.Add(w);          
  end;
  
  w:=new WeightingCombination('Weitzman:NoEW:Base2005');      
  w.Add(new GreenbookDiscounting(55));
  i_weightingCombinations.Add(w);          

  w:=new WeightingCombination('Weitzman:AvEW:Base2005');      
  w.Add(new GreenbookDiscounting(55));
  w.Add(new AverageEquityWeighting());
  i_weightingCombinations.Add(w);                      

  if not (Convert.ToInt32(parameters.emissionperiod)=2000) then
  begin
    var baseyear:=Convert.ToInt32(parameters.emissionperiod)-1950;
    
    for each prtp: Double in prtps do
    begin      
      w:=new WeightingCombination(String.Format('{0}%:NoEW:BaseEmission', prtp*100.0));      
      w.Add(new ConstantDiscounting(prtp,baseyear, String.Format('constant discount rate [prtp={0}%, base year=year of emission]', prtp)));
      i_weightingCombinations.Add(w);
      
      w:=new WeightingCombination(String.Format('{0}%:AvEW:BaseEmission', prtp*100.0));      
      w.Add(new AnthoffEquityWeighting(baseyear, '', 1.0));
      w.Add(new ConstantDiscountRate(prtp, baseyear));
      i_weightingCombinations.Add(w);
      
      w:=new WeightingCombination(String.Format('{0}%:WeuEW:BaseEmission', prtp*100.0));      
      w.Add(new AnthoffEquityWeighting(baseyear, 'WEU', 1.0));
      w.Add(new ConstantDiscountRate(prtp, baseyear));
      i_weightingCombinations.Add(w);          
    end;
    
    w:=new WeightingCombination('Weitzman:NoEW:BaseEmission');      
    w.Add(new GreenbookDiscounting(baseyear));
    i_weightingCombinations.Add(w);          

    w:=new WeightingCombination('Weitzman:AvEW:BaseEmission');      
    w.Add(new GreenbookDiscounting(baseyear));
    w.Add(new AverageEquityWeighting());
    i_weightingCombinations.Add(w);                      
  end;

  WeightingCombinations:=i_weightingCombinations.ToArray();
end;

procedure Fund28LegacyWeightingCombinations.EpaWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
var
  i_weightingCombinations:=new List<WeightingCombination>;
begin
  var prtps: array of double:=[0.0, 0.01, 0.03, 0.05];
  var riskaversions: array of double:=[0.5, 1.0, 1.5, 2.0, 2.5, 3.0];
  var consumptiondiscountrates:sequence of double:= [0.0175, 0.02, 0.03];
     
  var w: WeightingCombination;    
    
  for each prtp: Double in prtps do
  begin      
    for each riskaversion: double in riskaversions do
    begin         
      w:=new WeightingCombination(String.Format('ramsey:{0}%:{1}:NoEW:Base2005', prtp*100.0,riskaversion));      
      w.Add(new ConstantDiscounting(prtp,riskaversion,55));
      i_weightingCombinations.Add(w);
      
      w:=new WeightingCombination(String.Format('ramsey:{0}%:{1}:AvEW:Base2005', prtp*100.0,riskaversion));      
      w.Add(new AnthoffEquityWeighting(55, '', riskaversion));
      w.Add(new ConstantDiscountRate(prtp, 55));
      i_weightingCombinations.Add(w);
      
      w:=new WeightingCombination(String.Format('ramsey:{0}%:{1}:USAEW:Base2005', prtp*100.0,riskaversion));      
      w.Add(new AnthoffEquityWeighting(55, 'USA', riskaversion));
      w.Add(new ConstantDiscountRate(prtp, 55));
      i_weightingCombinations.Add(w);          
    end;
  end; 
  
  for each consumptiondiscountrate: Double in consumptiondiscountrates do
  begin
    w:=new WeightingCombination(String.Format('consumption:{0}%:NaN:NoEW:Base2005', consumptiondiscountrate*100.0));
    w.Add(new ConstantDiscountRate(consumptiondiscountrate, 55));
    i_weightingCombinations.Add(w);
  end;
  
  w:=new WeightingCombination('none:NaN:NaN:NoEW:NaN');
  w.Add(new UnityWeight());
  i_weightingCombinations.Add(w);  
  
  if not (Convert.ToInt32(parameters.emissionperiod)=2005) then
  begin
    var baseyear:=Convert.ToInt32(parameters.emissionperiod)-1950;
    
    for each prtp: Double in prtps do
    begin      
      for each riskaversion: double in riskaversions do
      begin         
        w:=new WeightingCombination(String.Format('ramsey:{0}%:{1}:NoEW:BaseEmission', prtp*100.0,riskaversion));      
        w.Add(new ConstantDiscounting(prtp,riskaversion,baseyear));
        i_weightingCombinations.Add(w);
        
        w:=new WeightingCombination(String.Format('ramsey:{0}%:{1}:AvEW:BaseEmission', prtp*100.0,riskaversion));      
        w.Add(new AnthoffEquityWeighting(baseyear, '', riskaversion));
        w.Add(new ConstantDiscountRate(prtp, baseyear));
        i_weightingCombinations.Add(w);
        
        w:=new WeightingCombination(String.Format('ramsey:{0}%:{1}:USAEW:BaseEmission', prtp*100.0,riskaversion));      
        w.Add(new AnthoffEquityWeighting(baseyear, 'USA', riskaversion));
        w.Add(new ConstantDiscountRate(prtp, baseyear));
        i_weightingCombinations.Add(w);          
      end;
    end; 
    
    for each consumptiondiscountrate: Double in consumptiondiscountrates do
    begin
      w:=new WeightingCombination(String.Format('consumption:{0}%:NaN:NoEW:BaseEmission', consumptiondiscountrate*100.0));
      w.Add(new ConstantDiscountRate(consumptiondiscountrate, baseyear));
      i_weightingCombinations.Add(w);
    end;
        
  end;
  
  for each wc in i_weightingCombinations do
  begin
    wc.Add(new ConstantFactorWeight(1.28, ''));
  end;
  
  WeightingCombinations:=i_weightingCombinations.ToArray();
end;

procedure Fund28LegacyWeightingCombinations.RawWeightingCombinations(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
var
  i_weightingCombinations:=new List<WeightingCombination>;
begin    
  var w: WeightingCombination;    
      
  w:=new WeightingCombination('RAW');
  w.Add(new UnityWeight());
  i_weightingCombinations.Add(w);  
    
  WeightingCombinations:=i_weightingCombinations.ToArray();
end;

procedure Fund28LegacyWeightingCombinations.RhoEtaSensitivityWeightingCombintaiontsPart1(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
var
  i_weightingCombinations:=new List<WeightingCombination>;
  i: integer;
  w: WeightingCombination;
begin    
  var prtps:=new List<double>;
  var etas:=new List<double> ;
  
  for i:=0 to 30 do
  begin
    prtps.Add(i*0.001);       
  end;
  
  for i:=0 to 15 do
  begin
    etas.Add(i*0.05);
  end;
    
  for each prtp: Double in prtps do
  begin      
    for each eta: Double in etas do
    begin
      w:=new WeightingCombination(String.Format('{0}:{1}:Global', prtp, eta));
      w.Add(new GlobalGrowthDiscounterateWeighting(55, eta));
      w.Add(new ConstantDiscountRate(prtp, 55));
      i_weightingCombinations.Add(w);           
      
      w:=new WeightingCombination(String.Format('{0}:{1}:NoEW', prtp, eta));
      w.Add(new ConstantDiscounting(prtp,eta,55));
      i_weightingCombinations.Add(w);
      
      w:=new WeightingCombination(String.Format('{0}:{1}:AvEW', prtp, eta));
      w.Add(new AnthoffEquityWeighting(55, '', eta));
      w.Add(new ConstantDiscountRate(prtp, 55));
      i_weightingCombinations.Add(w);
      
      w:=new WeightingCombination(String.Format('{0}:{1}:WeuEW', prtp, eta));
      w.Add(new AnthoffEquityWeighting(55, 'WEU', eta));
      w.Add(new ConstantDiscountRate(prtp, 55));
      i_weightingCombinations.Add(w);                
    end;
  end;
            
  WeightingCombinations:=i_weightingCombinations.ToArray();
end;

procedure Fund28LegacyWeightingCombinations.RhoEtaSensitivityWeightingCombintaiontsPart2(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
var
  i_weightingCombinations:=new List<WeightingCombination>;
  i: integer;
  w: WeightingCombination;
begin    
  var prtps:=new List<double>;
  var etas:=new List<double> ;
  
  for i:=0 to 30 do
  begin
    prtps.Add(i*0.001);       
  end;
  
  for i:=16 to 30 do
  begin
    etas.Add(i*0.05);
  end;
    
  for each prtp: Double in prtps do
  begin      
    for each eta: Double in etas do
    begin
      w:=new WeightingCombination(String.Format('{0}:{1}:Global', prtp, eta));
      w.Add(new GlobalGrowthDiscounterateWeighting(55, eta));
      w.Add(new ConstantDiscountRate(prtp, 55));
      i_weightingCombinations.Add(w);           
      
      w:=new WeightingCombination(String.Format('{0}:{1}:NoEW', prtp, eta));
      w.Add(new ConstantDiscounting(prtp,eta,55));
      i_weightingCombinations.Add(w);
      
      w:=new WeightingCombination(String.Format('{0}:{1}:AvEW', prtp, eta));
      w.Add(new AnthoffEquityWeighting(55, '', eta));
      w.Add(new ConstantDiscountRate(prtp, 55));
      i_weightingCombinations.Add(w);
      
      w:=new WeightingCombination(String.Format('{0}:{1}:WeuEW', prtp, eta));
      w.Add(new AnthoffEquityWeighting(55, 'WEU', eta));
      w.Add(new ConstantDiscountRate(prtp, 55));
      i_weightingCombinations.Add(w);                
    end;
  end;
            
  WeightingCombinations:=i_weightingCombinations.ToArray();
end;

procedure Fund28LegacyWeightingCombinations.RhoEtaSensitivityWeightingCombintaiontsPart3(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
var
  i_weightingCombinations:=new List<WeightingCombination>;
  i: integer;
  w: WeightingCombination;
begin    
  var prtps:=new List<double>;
  var etas:=new List<double> ;
  
  for i:=0 to 30 do
  begin
    prtps.Add(i*0.001);       
  end;
  
  for i:=31 to 45 do
  begin
    etas.Add(i*0.05);
  end;
    
  for each prtp: Double in prtps do
  begin      
    for each eta: Double in etas do
    begin
      w:=new WeightingCombination(String.Format('{0}:{1}:Global', prtp, eta));
      w.Add(new GlobalGrowthDiscounterateWeighting(55, eta));
      w.Add(new ConstantDiscountRate(prtp, 55));
      i_weightingCombinations.Add(w);           
      
      w:=new WeightingCombination(String.Format('{0}:{1}:NoEW', prtp, eta));
      w.Add(new ConstantDiscounting(prtp,eta,55));
      i_weightingCombinations.Add(w);
      
      w:=new WeightingCombination(String.Format('{0}:{1}:AvEW', prtp, eta));
      w.Add(new AnthoffEquityWeighting(55, '', eta));
      w.Add(new ConstantDiscountRate(prtp, 55));
      i_weightingCombinations.Add(w);
      
      w:=new WeightingCombination(String.Format('{0}:{1}:WeuEW', prtp, eta));
      w.Add(new AnthoffEquityWeighting(55, 'WEU', eta));
      w.Add(new ConstantDiscountRate(prtp, 55));
      i_weightingCombinations.Add(w);                
    end;
  end;
            
  WeightingCombinations:=i_weightingCombinations.ToArray();
end;

procedure Fund28LegacyWeightingCombinations.RhoEtaSensitivityWeightingCombintaiontsPart4(out WeightingCombinations: array of WeightingCombination; parameters: FundParameter);
var
  i_weightingCombinations:=new List<WeightingCombination>;
  i: integer;
  w: WeightingCombination;
begin    
  var prtps:=new List<double>;
  var etas:=new List<double> ;
  
  for i:=0 to 30 do
  begin
    prtps.Add(i*0.001);       
  end;
  
  for i:=46 to 60 do
  begin
    etas.Add(i*0.05);
  end;
    
  for each prtp: Double in prtps do
  begin      
    for each eta: Double in etas do
    begin
      w:=new WeightingCombination(String.Format('{0}:{1}:Global', prtp, eta));
      w.Add(new GlobalGrowthDiscounterateWeighting(55, eta));
      w.Add(new ConstantDiscountRate(prtp, 55));
      i_weightingCombinations.Add(w);           
      
      w:=new WeightingCombination(String.Format('{0}:{1}:NoEW', prtp, eta));
      w.Add(new ConstantDiscounting(prtp,eta,55));
      i_weightingCombinations.Add(w);
      
      w:=new WeightingCombination(String.Format('{0}:{1}:AvEW', prtp, eta));
      w.Add(new AnthoffEquityWeighting(55, '', eta));
      w.Add(new ConstantDiscountRate(prtp, 55));
      i_weightingCombinations.Add(w);
      
      w:=new WeightingCombination(String.Format('{0}:{1}:WeuEW', prtp, eta));
      w.Add(new AnthoffEquityWeighting(55, 'WEU', eta));
      w.Add(new ConstantDiscountRate(prtp, 55));
      i_weightingCombinations.Add(w);                
    end;
  end;
            
  WeightingCombinations:=i_weightingCombinations.ToArray();
end;
  
method Fund28LegacyWeightingCombinations.GetWeightingCombinationsFromName(name: String; out i_WeightingCombinations: array of WeightingCombination; parameters: FundParameter); 
begin
  case name of
    'Standard': StandardWeightingCombinations(out i_weightingCombinations, parameters);
    'Jiehan': JiehansWeightingCombinations(out i_weightingCombinations, parameters);
    'Ada': AdaWeightingCombinations(out i_weightingCombinations, parameters);
    'David': DavidWeightingCombinations(out i_weightingCombinations, parameters);
    'Defra': DefraWeightingCombinations(out i_weightingCombinations, parameters);
    'Paul': PaulWeightingCombinations(out i_weightingCombinations, parameters);
    'Cameron': CameronWeightingCombinations(out i_weightingCombinations, parameters);
    'Megan': MeganWeightingCombinations(out i_weightingCombinations, parameters);
    'Dritan': DritanWeightingCombinations(out i_weightingCombinations, parameters);
    'Needs': NeedsWeightingCombinations(out i_weightingCombinations, parameters);
    'Epa': EpaWeightingCombinations(out i_weightingCombinations, parameters);
    'Raw': RawWeightingCombinations(out i_weightingCombinations, parameters);
    'RhoEtaSensitivityPart1': RhoEtaSensitivityWeightingCombintaiontsPart1(out i_weightingCombinations, parameters);
    'RhoEtaSensitivityPart2': RhoEtaSensitivityWeightingCombintaiontsPart2(out i_weightingCombinations, parameters);
    'RhoEtaSensitivityPart3': RhoEtaSensitivityWeightingCombintaiontsPart3(out i_weightingCombinations, parameters);
    'RhoEtaSensitivityPart4': RhoEtaSensitivityWeightingCombintaiontsPart4(out i_weightingCombinations, parameters);    
    else raise new ApplicationException('Invalid weighting scheme');
  end;      
end;

end.
