// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

type
  // This class implements an equity weighting scheme based on average income:
  // equity weight = average world per capita income / average regional per capiat income
  AverageEquityWeighting = public class( WeightingScheme )
  protected
    function GetName: string; override;
  public
    procedure CalculateWeights(ModelOutput: ModelOutput); override;
  end;
  
implementation

{ AverageEquityWeighting }

procedure AverageEquityWeighting.CalculateWeights(
  ModelOutput: ModelOutput);
var
  i_worldIncome, i_worldPopulation: double;
  i_equityWeight: double;
  i, l: Integer;
begin
  inherited;
  // TODO -oDavid Anthoff: Change start of "real" period to constant
  for i:=50 to LegacyConstants.NYear do
  begin
    i_worldIncome:=0;
    i_worldPopulation:=0;

    for l:=1 to LegacyConstants.NoReg do
    begin
      i_worldIncome:=i_worldIncome + ModelOutput.Incomes.Item[i, l];
      i_worldPopulation:=i_worldPopulation + ModelOutput.Populations.Item[i, l];
    end;

    for l:=1 to LegacyConstants.NoReg do
    begin
      i_equityWeight:=(i_worldIncome/i_worldPopulation)/(ModelOutput.Incomes.Item[i, l]/ModelOutput.Populations.Item[i, l]);
      AddWeight(i, l, i_equityWeight);
    end;
  end;
end;

function AverageEquityWeighting.GetName: string;
begin
  Result:='average equity weighting';
end;

end.