// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

type
  ConstantDiscountRate = public class (WeightingScheme)
  private
    m_rate: double;
    m_name: string;
    m_baseYear: Integer;
  protected
    method GetName: string; override;
    method GetWeight(Year: Integer; Region: integer): double; override;
  public
    constructor(discountRate: double);
    constructor(discountRate: Double; baseYear: Integer);
    constructor(discountRate: double; name: String);

    method CalculateWeights(ModelOutput: ModelOutput); override;
  end;
  
implementation

method ConstantDiscountRate.GetName: string; 
begin
  Result:=String.Format('{0}={1}%', m_name, m_rate*100);
end;

method ConstantDiscountRate.GetWeight(Year: Integer; Region: integer): double; 
begin
  if Year<m_baseYear then
    raise new Exception('this is not supported');
    
  Result:=1/Math.Pow(1 + m_rate, Year-m_baseYear);
end;

constructor ConstantDiscountRate(discountRate: double); 
begin
  constructor(discountRate, 'discount rate');
end;

method ConstantDiscountRate.CalculateWeights(ModelOutput: ModelOutput); 
begin
end;

constructor ConstantDiscountRate(discountRate: double; name: String); 
begin
  m_rate:=discountRate;
  m_name:=name;
  m_baseYear:=50;
end;

constructor ConstantDiscountRate(discountRate: Double; baseYear: Integer); 
begin
  m_rate:=discountRate;
  m_baseYear:=baseYear;
end;

end.