// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

type
  WeitzmanStepDecline = public class (WeightingScheme)
  protected
    function GetName: string; override;
  public
    constructor;

    procedure CalculateWeights(ModelOutput: ModelOutput); override;
  end;
  
implementation

{ WeitzmanStepDecline }

procedure WeitzmanStepDecline.CalculateWeights(ModelOutput: ModelOutput);
var
  i, l: Integer;
  i_discountFactor: double;
  i_discountRate: double;
begin
  inherited;
  i_discountFactor:=1;
  // TODO -oDavid Anthoff: Change start of "real" period to constant
  for i:=50 to LegacyConstants.NYear do
  begin
    for l:=1 to LegacyConstants.NoReg do
      AddWeight(i, l, i_discountFactor);

    if i<55 then
      i_discountRate:=0.04
    else if i<75 then
      i_discountRate:=0.03
    else if i<125 then
      i_discountRate:=0.02
    else if i<350 then
      i_discountRate:=0.01
    else
      i_discountRate:=0.00;
    i_discountFactor:= i_discountFactor/(1.00 + i_discountRate);
  end;
end;

constructor WeitzmanStepDecline;
begin
  inherited constructor;
end;

function WeitzmanStepDecline.GetName: string;
begin
  Result:='Weitzman step decline';
end;

end.